/********************************** (C) COPYRIGHT *******************************
 * File Name          : CH59x_SYS.h
 * Author             : WCH
 * Version            : V1.2
 * Date               : 2021/11/17
 * Description
 *********************************************************************************
 * Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
 * Attention: This software (modified or not) and binary are used for 
 * microcontroller manufactured by Nanjing Qinheng Microelectronics.
 *******************************************************************************/

#ifndef __CH59x_SYS_H__
#define __CH59x_SYS_H__

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief  rtc interrupt event define
 */
typedef enum
{
    RST_STATUS_SW = 0, // λ
    RST_STATUS_RPOR,   // ϵ縴λ
    RST_STATUS_WTR,    // Źʱλ
    RST_STATUS_MR,     // ⲿֶλ
    RST_STATUS_LRM0,   // Ѹλ-λ
    RST_STATUS_GPWSM,  // µģʽѸλ
    RST_STATUS_LRM1,   //	Ѹλ-Ź
    RST_STATUS_LRM2,   //	Ѹλ-ֶλ

} SYS_ResetStaTypeDef;

/**
 * @brief  rtc interrupt event define
 */
typedef enum
{
    INFO_ROM_READ = 0, // FlashROM  Ƿɶ
    INFO_RESET_EN = 2, // RST#ⲿֶλ빦Ƿ
    INFO_BOOT_EN,      // ϵͳ BootLoader Ƿ
    INFO_DEBUG_EN,     // ϵͳԽӿǷ
    INFO_LOADER,       // ǰϵͳǷBootloader 
    STA_SAFEACC_ACT,   // ǰϵͳǷڰȫ״̬RWA򲻿ɷ

} SYS_InfoStaTypeDef;

/**
 * @brief  ȡоƬID࣬һΪ̶ֵ
 */
#define SYS_GetChipID()      R8_CHIP_ID

/**
 * @brief  ȡȫIDһΪ̶ֵ
 */
#define SYS_GetAccessID()    R8_SAFE_ACCESS_ID

/**
 * @brief   ϵͳʱ
 *
 * @param   sc      - ϵͳʱԴѡ refer to SYS_CLKTypeDef
 */
void SetSysClock(SYS_CLKTypeDef sc);

/**
 * @brief   ȡǰϵͳʱ
 *
 * @return  Hz
 */
uint32_t GetSysClock(void);

/**
 * @brief   ȡǰϵͳϢ״̬
 *
 * @param   i       - refer to SYS_InfoStaTypeDef
 *
 * @return  Ƿ
 */
uint8_t SYS_GetInfoSta(SYS_InfoStaTypeDef i);

/**
 * @brief   ȡϵͳϴθλ״̬
 *
 * @return  refer to SYS_ResetStaTypeDef
 */
#define SYS_GetLastResetSta()    (R8_RESET_STATUS & RB_RESET_FLAG)

/**
 * @brief   ִϵͳλ
 */
void SYS_ResetExecute(void);

/**
 * @brief   øλĴֵֶλ λ ŹλͨѸλӰ
 *
 * @param   i       - refer to SYS_InfoStaTypeDef
 */
#define SYS_ResetKeepBuf(d)    (R8_GLOB_RESET_KEEP = d)

/**
 * @brief   رжϣǰжֵ
 *
 * @param   pirqv   - ǰжֵ
 */
void SYS_DisableAllIrq(uint32_t *pirqv);

/**
 * @brief   ָ֮ǰرյжֵ
 *
 * @param   irq_status  - ǰжֵ
 */
void SYS_RecoverIrq(uint32_t irq_status);

/**
 * @brief   ȡǰϵͳ(SYSTICK)ֵ
 *
 * @return  ǰֵ
 */
uint32_t SYS_GetSysTickCnt(void);

/**
 * @brief   ؿŹֵ
 *
 * @param   c       - Źֵ
 */
#define WWDG_SetCounter(c)    (R8_WDOG_COUNT = c)

/**
 * @brief   Źʱжʹ
 *
 * @param   s       - Ƿж
 */
void WWDG_ITCfg(FunctionalState s);

/**
 * @brief   Źʱλ
 *
 * @param   s       - Ƿλ
 */
void WWDG_ResetCfg(FunctionalState s);

/**
 * @brief   ȡǰŹʱ־
 *
 * @return  Źʱ־
 */
#define WWDG_GetFlowFlag()    (R8_RST_WDOG_CTRL & RB_WDOG_INT_FLAG)

/**
 * @brief   Źжϱ־¼ؼֵҲ
 */
void WWDG_ClearFlag(void);

/**
 * @brief   uS ʱ
 *
 * @param   t       - ʱ
 */
void mDelayuS(uint16_t t);

/**
 * @brief   mS ʱ
 *
 * @param   t       - ʱ
 */
void mDelaymS(uint16_t t);

extern volatile uint32_t IRQ_STA;

/**
 * @brief Enter safe access mode.
 * 
 * @NOTE: After enter safe access mode, about 16 system frequency cycles 
 * are in safe mode, and one or more secure registers can be rewritten 
 * within the valid period. The safe mode will be automatically 
 * terminated after the above validity period is exceeded.
 */
 __attribute__((always_inline)) static inline void sys_safe_access_enable(void)
{
    if(read_csr(0x800)&0x08)
    {
        IRQ_STA = read_csr(0x800);
        write_csr(0x800, (IRQ_STA&(~0x08)));
    }
    SAFEOPERATE;
    R8_SAFE_ACCESS_SIG = SAFE_ACCESS_SIG1;
    R8_SAFE_ACCESS_SIG = SAFE_ACCESS_SIG2;
    SAFEOPERATE;
}

__attribute__((always_inline)) static inline void sys_safe_access_disable(void)
{
    R8_SAFE_ACCESS_SIG = 0;
    write_csr(0x800, read_csr(0x800)|(IRQ_STA&0x08));
    IRQ_STA = 0;
    SAFEOPERATE;
}

#ifdef __cplusplus
}
#endif

#endif // __CH59x_SYS_H__
